/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package dbfastclock;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.MulticastSocket;
import java.net.UnknownHostException;
import java.util.Arrays;

/**
 *
 * @author Daniel Bergqvist
 */
public class MulticastClient implements Runnable
{
    MySignal mySignal;
	
    public MulticastClient(MySignal aMySignal)
	{
        mySignal = aMySignal;
    }
    
	@Override
    public void run()
	{
	    String sMulticastAddress = mySignal.getMulticastAddress();
	    int iMulticastPort = mySignal.getMulticastPort();
		
		InetAddress mcastAddr = null;

		try
		{
			mcastAddr = InetAddress.getByName(sMulticastAddress);
		}
		catch (UnknownHostException uhe)
		{
			System.out.println("Problems getting the symbolic multicast address");

			uhe.printStackTrace();
			System.exit(1);
		}
		
		MulticastSocket mSocket = null;
		
		try
		{
			System.out.println("Setting up multicast receiver");
			
			mSocket = new MulticastSocket(iMulticastPort);
			
			mSocket.joinGroup(mcastAddr);
		}
		catch (IOException ioe)
		{
			System.out.println("Trouble opening multicast port");
			
			ioe.printStackTrace();
			System.exit(1);
		}
		
		DatagramPacket packet;
		
		System.out.println("Multicast receiver set up ");
		
		while (true)
		{
			try
			{
				byte[] buf = new byte[1000];
				
				packet = new DatagramPacket(buf, buf.length);
				
				//System.out.println("McastReceiver: waiting for packet");
				mSocket.receive(packet);
				
				byte data[] = Arrays.copyOf(packet.getData(),packet.getLength());
				
				// Convert byte array to string, but only that number of bytes that has been received
//				String message = new String(data).substring(0, packet.getLength());
				String message = new String(data,"UTF8");
				
				parseMessage(message);
				
				System.out.println(message);
			}
			catch (IOException e)
			{
				System.out.println("Trouble reading multicast message");

				e.printStackTrace();
				System.exit(1);

			}
		}

	}
	
	
	public void parseMessage(String message)
	{
		int iType = 0;
		boolean bIsActive = false;
		int iDay = -1;
		int iHour = -1;
		int iMin = -1;
		int iSec = -1;
		double dSpeed = 1.0;
		String sTextMessage = "";
		int iEqualSignIndex;
		String sParameter;
		String sValue;
		boolean bTextMessageVisible = false;
        
		String lines[] = message.split("\r\n");
		String sLine;
		
		String nameOfClock = "";
		
		try
		{
//			while ((sLine = in.readLine()) != null)
			for (int iLineNr=0; iLineNr < lines.length; iLineNr++)
			{
				sLine = lines[iLineNr];

				if (iLineNr == 0)	// Version 1 of the protocol
				{
					if (!sLine.equals("fastclock"))
					{
						System.out.println("Error: Packet is not a FastClock packet");
						return;
					}
				}
				else	// Version 2 of the protocol
				{
					iEqualSignIndex = sLine.indexOf("=");
					
					if (iEqualSignIndex != -1)
					{
						sParameter = sLine.substring(0, iEqualSignIndex);
						sValue = sLine.substring(iEqualSignIndex + 1);
						
//						System.out.println("Parameter: "+sParameter+", value: "+sValue+"\n");
						
						switch (sParameter)
						{
							case "clocktype":
								
								if ("fastclock".equals(sValue))
								{
									iType = MySignal.CLOCK_TYPE__FAST_CLOCK;
								}
								else
								{
									if ("realclock".equals(sValue))
									{
										iType = MySignal.CLOCK_TYPE__REAL_CLOCK;
									}
									else
									{
										mySignal.debug.write("Error: Server send wrong clock type: " + sParameter + "\n");
									}
								}
								break;
								
							case "active":
								
								if ("yes".equals(sValue))
								{
									bIsActive = true;
								}
								else
								{
									if ("no".equals(sValue))
									{
										bIsActive = false;
									}
									else
									{
										mySignal.debug.write("Error: Server send wrong \"active\": " + sParameter + "\n");
									}
								}
								break;
								
							case "day":
								
								switch (sValue)
								{
									case "none":
										iDay = MySignal.WEEKDAY_NONE;
										break;
									case "monday":
										iDay = MySignal.WEEKDAY_MONDAY;
										break;
									case "tuesday":
										iDay = MySignal.WEEKDAY_TUESDAY;
										break;
									case "wednesday":
										iDay = MySignal.WEEKDAY_WEDNESDAY;
										break;
									case "thursday":
										iDay = MySignal.WEEKDAY_THURSDAY;
										break;
									case "friday":
										iDay = MySignal.WEEKDAY_FRIDAY;
										break;
									case "saturday":
										iDay = MySignal.WEEKDAY_SATURDAY;
										break;
									case "sunday":
										iDay = MySignal.WEEKDAY_SUNDAY;
										break;
									default:
										mySignal.debug.write("Error: Server send wrong weekday: " + sParameter + "\n");
										break;
								}
								break;
								
							case "clock":
								
								String[] sValues = sValue.split(":");
								
								try
								{
									iHour = Integer.parseInt(sValues[0].trim());
									iMin = Integer.parseInt(sValues[1].trim());
									iSec = Integer.parseInt(sValues[2].trim());
								}
								catch (NumberFormatException ex)
								{
									mySignal.debug.write("Exception: " + ex.getMessage() + ". Parameter 'clock', value: '" + sValue + "''");
								}
								break;
								
							case "speed":
								dSpeed = Double.parseDouble(sValue);
								break;
								
							case "text":
								sTextMessage = sValue;
								sTextMessage = sTextMessage.replaceAll("\\\\n", "\n");
								// Visa text om textmeddelandet inte är tomt
								bTextMessageVisible = !sTextMessage.equals("");
								break;
								
							case "name":
								nameOfClock = sValue;
								mySignal.addClock(nameOfClock);
								break;
								
						}
					}
					else
					{
						if (!sLine.equals(""))
						{
							mySignal.debug.write("Error: Server send wrong string. No equal sign: " + sLine + "\n");
						}
					}
				}
			}
			
			if (nameOfClock.equals(mySignal.getNameOfClock()))
			{
				mySignal.setClock(iType, bIsActive, iDay, iHour, iMin, iSec, dSpeed);
				mySignal.SetText(bTextMessageVisible, sTextMessage);
			}
		}
		catch (IOException ex)
		{
			System.out.println(ex.getMessage());
		}
	}
	
	
}
